#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include "LEPC.h"

//Auto running time (sec.)
#define RUN_TIME 30

//Max message size to print
#define MAX_BUFSIZE 100

// Datafile pointer
FILE *gDataFile;

/// User defined dump format ///

void PrintHeader(){
	if(!gDataFile) return;
	fprintf(gDataFile,"kind,time,SD(Hex),SD(char),RD(Hex),RD(char)\n");
	fflush(gDataFile);
}

void PrintIdle(float idle){
	if(!gDataFile) return;
	fprintf(gDataFile,"IDLE(sec.),%.2f,,\n", idle);
	fflush(gDataFile);
}

void PrintTimeSD(int hv, int mv, int lv){
	if(!gDataFile) return;
	fprintf(gDataFile,"TIME(SD),%02dm %02d.%02ds,,\n", hv, mv, lv);
	fflush(gDataFile);
}

void PrintTimeRD(int hv, int mv, int lv){
	if(!gDataFile) return;
	fprintf(gDataFile,"TIME(RD),%02dm %02d.%02ds,,\n", hv, mv, lv);
	fflush(gDataFile);
}

void PrintDataSDRD(int sdc, int rdc, unsigned char *d_SD, unsigned char *d_RD){
	int i;
	if(!gDataFile) return;
	fprintf(gDataFile,",,");
	for(i =0; i<sdc; i++){
		fprintf(gDataFile,"%02x", d_SD[i]);
	}
	fprintf(gDataFile,",");
	for(i =0; i<sdc; i++){
		if(d_SD[i]>31 && d_SD[i]<127) // normal ascii chars
			fprintf(gDataFile,"%c", d_SD[i]);
	}
	fprintf(gDataFile,",");
	for(i =0; i<rdc; i++){
		fprintf(gDataFile,"%02x", d_RD[i]);
	}
	fprintf(gDataFile,",");
	for(i =0; i<rdc; i++){
		if(d_RD[i]>31 && d_RD[i]<127) // normal ascii chars
			fprintf(gDataFile,"%c", d_RD[i]);
	}
	fprintf(gDataFile,"\n");
	fflush(gDataFile);
}


// ********************************************************
// ** CallBack
// ********************************************************
void CallBackFunc(void *Buf, int datasize)
{
	int i;
	LE_MONDATAFORMAT2 *data = (LE_MONDATAFORMAT2 *)Buf;
	int sdtime;
	int rdtime;
	int sdcount = 0;
	int rdcount = 0;
	unsigned char sdbuf[MAX_BUFSIZE];
	unsigned char rdbuf[MAX_BUFSIZE];

	for( i = 0; i < datasize/(int)sizeof(LE_MONDATAFORMAT2); i ++ ){
		// time
		if( (data[i].framestatus & 0x3) == LE_MONSTATUS_IDLETIME ){
			// IDLE TIME
			PrintIdle( (data[i].time.l+data[i].time.m*0x100+data[i].time.h*0x10000) * 0.01 ); // 10ms unit
		}
		if( (data[i].framestatus & 0x3) == LE_MONSTATUS_SD_TIMESTAMP ){
			// TIMESTAMP SD
			PrintTimeSD( data[i].time.h, data[i].time.m, data[i].time.l );
		}
		if( (data[i].framestatus & 0x3) == LE_MONSTATUS_RD_TIMESTAMP ){
			// TIMESTAMP RD
			PrintTimeRD( data[i].time.h, data[i].time.m, data[i].time.l );
		}

		// data
		if(data[i].framestatus & LE_MONSTATUS_SD_DATA_VISIBLE ){
			// SD data
			if(sdcount < MAX_BUFSIZE){
				sdbuf[sdcount] = data[i].data.SD;
				sdcount ++;
			}
		}
		if(data[i].framestatus & LE_MONSTATUS_RD_DATA_VISIBLE ){
			// RD data
			if(rdcount < MAX_BUFSIZE){
				rdbuf[rdcount] = data[i].data.RD;
				rdcount ++;
			}
		}
	}
	PrintDataSDRD(sdcount, rdcount, sdbuf, rdbuf);
}

// controls //

int Start()
{
	if( LEStartOnlineCallBack(CallBackFunc, 2) == LE_FAIL ){
		return -1;
		printf("LEStartOnlineCallBack failed\n");
	}
	printf("LEStartOnlineCallBack success\n");
	return 0;
}

int Stop()
{
	LEStop();
	printf("LEStop\n");
	return 0;
}

// user functions //

int GetRunNo(const char * fname){
  FILE *f_run;
  int runno = 1;
  char buf[100];
  
  // read setting from file
  if((f_run = fopen(fname,"r")) == NULL){
    if((f_run = fopen(fname,"w+")) == NULL){
      printf("Cannot read/create runNo file, check permission\n");
      return -1;
    }
    runno = 1;
  }
  else if((fgets(buf,100,f_run)) != NULL){
    sscanf(buf, "%d", &runno);
  }
  if(runno<=1 || runno>999999){
    runno = 1;
    printf("run No. has not saved or invalid. auto set to 1\n");
  }
  else
    printf("Next run No. = %d : loaded\n",runno);

  fclose(f_run);
  
  return runno;
}

int PutRunNo(int runno, const char * fname){
  FILE *f_run;
  char buf[100];

  // save current runno. //
  
  if((f_run = fopen(fname,"w+")) == NULL){
	  printf("Cannot create runNo file, check permission\n");
      return -1;
  }

  fprintf(f_run,"%d\n",runno);

  fclose(f_run);
  
  return runno;
}


int StartRun(int runno, const char * runfname){
  FILE *fr;
  char fname[100];
  time_t tm_start;
  time_t tm_now;
  struct tm *tmexp;
  
  if((fr = fopen(runfname, "w+")) == NULL){
    printf("cannot update runno file\n");
    return -1;
  }
  fprintf(fr, "%d\n", runno);
  fclose(fr);
  
  sprintf(fname, "run%06d.csv", runno);
  if((gDataFile = fopen(fname, "w+")) == NULL){
    printf("cannot open datafile %s \n",fname);
    return -1;
  }

  printf("Data file name = %s\n",fname);

  PrintHeader();
  
  tm_now = time(NULL);
  tm_start = tm_now;
  tmexp = localtime(&tm_start);
  printf("Start Time (PC) = %02d/%02d %02d:%02d:%02d\n",
	 tmexp->tm_mon+1,
	 tmexp->tm_mday,
	 tmexp->tm_hour,
	 tmexp->tm_min,
	 tmexp->tm_sec
	 );

  // start with a check//
  if(Start() == -1) {
	  fclose(gDataFile);
	  return -1;
  }
  while( (tm_now - tm_start) < RUN_TIME ){
    sleep(1);
    tm_now = time(NULL);
    printf("\r %02ld / %d sec. elapsed", (tm_now - tm_start), RUN_TIME);
    fflush(stdout);
  }
  tmexp = localtime(&tm_now);
  printf("\nEnd Time (PC) = %02d/%02d %02d:%02d:%02d\n",
	 tmexp->tm_mon+1,
	 tmexp->tm_mday,
	 tmexp->tm_hour,
	 tmexp->tm_min,
	 tmexp->tm_sec
	 );
  
  
  Stop();
  
  fclose(gDataFile);
  return 0;
}

void PrintMsg(int runno){
  printf("\n-------------------\n");
  printf("Next Runno = %d\n",runno);
  printf("-------------------\n");
  printf("Enter command\n");
  printf("[R/r]:run  [Q/q]:quit \n");
}

int main(int argc, char *argv[])
{
	int i;
	int runno = 1;
	const char * runno_text = "runnum.txt";

	if( argc == 1){
		// connect without serial
		if( LEConnect("") == LE_FAIL ) 	{
			printf("LEConnect failed\n");
			return -1;
		}
		printf("LEConnect (Auto)\n");
	}
	else if(argc == 2){
		// connect with serial
		if( LEConnect(argv[1]) == LE_FAIL ) 	{
			printf("LEConnect failed\n");
			return -1;
		}
		printf("LEConnect %s\n",argv[1]);
	}
	else{
		printf("usage : RUNandDUMP <serial number>\n");
		return 0;
	}

	// load setting
	runno = GetRunNo(runno_text);

	// LE setting
	if( LESetTimeStampRec(LE_TIMESTAMPREC_MS10ms) == LE_FAIL ) 	{
		printf("Time stamp setting failed\n");
	}
	else
		printf("Time stamp type set to Min:Sec:10ms\n");
			
	if( LESetIdleTimeRec(LE_IDLETIMEREC_10ms) == LE_FAIL ) 	{
		printf("Idle time setting failed\n");
	}
	else
		printf("Idle time unit set to 10ms\n");
	// main loop

	PrintMsg(runno);
	while( (i = getchar()) != EOF){
	  switch(i){
	  case 'R':
	  case 'r':
	    printf("Run %d start\n",runno);
	    if(StartRun(runno, runno_text) != 0){
	      printf("start run failed\n");
	      PrintMsg(runno);
	      break;
	    }
	    // next
	    runno ++;
		PutRunNo(runno, runno_text);
	    PrintMsg(runno);
	    break;
	  case 'Q':
	  case 'q':
	    printf("quit\n");
	    LEDisconnect();
	    return 0;
	    break;
	  }
	}

	LEDisconnect();
	return 0;
}
